#ifndef __CHARACTERS_H__
#define __CHARACTERS_H__
#include <vector>
#include <string>
#include <memory>
#include <math.h>

#include "constants.h"
#include "display.h"
#include "fraction.h"
#include "position.h"
#include "potions.h"
#include "gold.h"
#include "rng.h"

class character; // forward declaration

// Note: player should not be in the character list


class character {
public:
        character(RNG *rng, const race &nrace,
                  const position &pos); // fills out the starting stats

        // usually I wouldn't do this but considering that the map has
        //  a super small size an O(n) solution is acceptable
        // IMPORTANT: available_positions do NOT have characters in them
        direction_list moveable(const position_list &available_positions) const;
        virtual result move(const direction dir,
                            const position_list &available_positions);

        struct attack_result {
                result res;
                std::string msg;
        };

        struct apply_result {
                result res;
                std::string msg;
        };
        virtual result move(position_list spots, const direction &dir);

        virtual attack_result attack(const direction dir, character *ch) = 0;

        virtual apply_result apply(const direction &dir,
                                   potion_list &potions);
        virtual attack_result get_hit(const enum race &race, const int atk,
                                      const fraction hitrate) = 0;

        // overload for different races
        virtual void print(display *out);

        virtual result calc_effects();
        void discard_level_effects();
        virtual void start_turn();

        enum race get_race() const;
        position get_pos() const;
        void set_pos(const position &npos);

        int get_HP() const;
        int get_ATK() const;
        int get_DEF() const;
        fraction get_hitrate() const;
        int get_room_num() const;
        void set_HP(const int nHP);
        void set_ATK(const int nATK);
        void set_DEF(const int nDEF);
        void set_hitrate(const fraction nhitrate);
        void set_room_num(const int room);

protected:
        RNG *rng;

        const feature enabled_features;
        const enum race race;

        int HP;
        position pos;

        // IMPORTANT: keep track of ATK and DEF in game at turn time
        int ATK;
        int DEF;
        fraction base_hit_rate;

        potion_list effects;

        int calc_dmg(const int ATK, const int DEF);
private:
        void insert_effect(potion *effect);
};

typedef std::vector<character *> character_list;

#endif
